"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _objectCreate = require("./object-create");

var _objectCreate2 = _interopRequireDefault(_objectCreate);

var _validate2 = require("./validate");

var _validate3 = _interopRequireDefault(_validate2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Validation fields definition.
 * @class Fields
 */

var FieldSet = function () {
  /**
   * @class Fields
   * @constructor
   * @param {Wildgeese} wildgeese     instance of wildgeese
   */

  function FieldSet(wildgeese) {
    _classCallCheck(this, FieldSet);

    this._fields = (0, _objectCreate2.default)();
    this._wildgeese = wildgeese;
  }

  /**
   * @return {FieldSet}
   */

  _createClass(FieldSet, [{
    key: "clone",
    value: function clone() {
      var f = new FieldSet(this._wildgeese);
      f._fields = _lodash2.default.cloneDeep(this._fields);
      return f;
    }

    /**
     * @method add
     * @param {String} fieldName        name of adding field
     * @param {String} label            Human readable field label
     * @param {Array<String|Function>}  validation rule names
     */

  }, {
    key: "add",
    value: function add(fieldName, label, rules) {
      label = label || fieldName;
      this._fields[fieldName] = { name: fieldName, label: label, rules: rules };
    }

    /**
     * @method remove
     * @param {String} fieldName        name of field to remove
     */

  }, {
    key: "remove",
    value: function remove(fieldName) {
      delete this._fields[fieldName];
    }

    /**
     * @return {Object}
     */

  }, {
    key: "fields",
    value: function fields() {
      return _lodash2.default.cloneDeep(this._fields);
    }

    /**
     * @method field
     * @param {String} fieldName        name of field to get
     * @return {Object}
     */

  }, {
    key: "field",
    value: function field(fieldName) {
      return _lodash2.default.cloneDeep(this._fields[fieldName]);
    }

    /**
     * @method validate
     * @param {Object} values                   values to validation <fieldName : value>
     * @param {Booelan} presentOnly             validate only presented fields
     * @return {Promise} reject with error messages Object<fieldName: Array<String>>
     */

  }, {
    key: "validate",
    value: function validate(values) {
      var presentOnly = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];

      return (0, _validate3.default)(this, values, this._wildgeese, presentOnly);
    }
  }]);

  return FieldSet;
}();

exports.default = FieldSet;
module.exports = exports['default'];