"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.validateValue = undefined;

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _co = require("co");

var _co2 = _interopRequireDefault(_co);

var _resolveValidator = require("./resolve-validator");

var _resolveValidator2 = _interopRequireDefault(_resolveValidator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } step("next"); }); }; }

var nextTick = function () {
    var ref = _asyncToGenerator(function* () {
        return new Promise(function (resolve) {
            return setTimeout(resolve, 0);
        });
    });

    return function nextTick() {
        return ref.apply(this, arguments);
    };
}();

/**
 * @param {FieldSet} fieldSet
 * @param {Object} values
 * @param {Wildgeese} wildgeese
 * @param {Boolean|Array<String>|String} presentOnly  when true only validate presented fields.
 * @return {Promise}        reject with error messages Object<fieldName: Array<String>>
 */

exports.default = function () {
    var ref = _asyncToGenerator(function* (fieldSet, values, wildgeese) {
        var presentOnly = arguments.length <= 3 || arguments[3] === undefined ? false : arguments[3];

        var fields = fieldSet.fields();
        var errors = {};
        var willValidateFields;

        // decide validate fields
        switch (true) {
            case Array.isArray(presentOnly):
                willValidateFields = presentOnly;break;
            case presentOnly === true:
                willValidateFields = Object.keys(values);break;
            case typeof presentOnly === "string":
                willValidateFields = [presentOnly];break;
            default:
                willValidateFields = Object.keys(fields);
        }

        //
        // make getters
        //
        var labelGetter = {};
        Object.keys(fields).forEach(function (fieldName) {
            var field = fields[fieldName];

            Object.defineProperty(labelGetter, fieldName, {
                enumerable: true,
                value: field.label
            });
        });

        // Make read only values accessor for validator
        var valueGetter = {};
        Object.keys(fields).forEach(function (fieldName) {
            Object.defineProperty(valueGetter, fieldName, {
                enumerable: true,
                value: values[fieldName]
            });
        });

        //
        // Validation
        //
        var hasFailed = false;

        try {
            yield Promise.all(willValidateFields.map(function (fieldName) {
                return new Promise(function () {
                    var ref = _asyncToGenerator(function* (resolve, reject) {
                        yield nextTick();

                        var fail = false;
                        var field = fields[fieldName];
                        var value = values[fieldName];

                        try {
                            var fieldErrors = yield validateValue(value, field.rules, wildgeese, {
                                label: field.label,
                                labels: labelGetter,
                                values: valueGetter
                            });

                            if (fieldErrors && fieldErrors.length > 0) {
                                var _errors$fieldName;

                                errors[fieldName] = errors[fieldName] || [];
                                (_errors$fieldName = errors[fieldName]).push.apply(_errors$fieldName, _toConsumableArray(fieldErrors));
                                fail = true;
                            }

                            resolve();
                        } catch (e) {
                            reject(e);
                            fail = true;
                        }

                        hasFailed = hasFailed || fail;
                    });

                    return function (_x5, _x6) {
                        return ref.apply(this, arguments);
                    };
                }());
            }));
        } catch (e) {
            throw e;
        }

        if (hasFailed) {
            return errors;
        } else {
            return;
        }
    });

    return function validateFields(_x2, _x3, _x4) {
        return ref.apply(this, arguments);
    };
}();

/**
 * @param {Any} value
 * @param {Array<String|Function>} rules
 * @param {Wildgeese} wildgeese
 * @param {Object} ctx
 * @param {String} ctx.label  human readable label(field name)
 * @param {Object} ctx.fields  other field values
 * @param {Object} ctx.label  other field labels
 * @return {Promise}        reject with error messages Array<String>
 */

var validateValue = exports.validateValue = function () {
    var ref = _asyncToGenerator(function* (value, rules, wildgeese) {
        var ctx = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

        var requiredValidator = wildgeese.getRule("required");
        var isOptional = rules.indexOf("required") === -1;

        // Normalize ctx
        ctx.labels = ctx.labels || {};
        ctx.values = ctx.values || {};
        ctx.options = ctx.options || {};

        // clone rules
        rules = _lodash2.default.cloneDeep(rules);

        var results = yield Promise.all(rules.map(function (rule) {
            return new Promise(function () {
                var ref = _asyncToGenerator(function* (resolve, reject) {
                    yield nextTick();

                    try {
                        var ruleName;
                        var validator;
                        var args;

                        // parse rule
                        if (Array.isArray(rule)) {
                            // Case of rule is [<String|Function>, ...options];
                            validator = rule.shift();
                            args = rule.shift();
                        } else {
                            // Case of rule is <String|Function>
                            validator = rule;
                        }

                        ruleName = typeof validator === "function" ? validator.name : validator;
                        args = args || {};

                        // Resolve validator
                        validator = (0, _resolveValidator2.default)(wildgeese, validator);

                        // validate
                        var givenCtx = {
                            args: args,
                            label: ctx.label,
                            labels: ctx.labels,
                            options: wildgeese.get(),
                            values: ctx.values
                        };

                        // Ignore validation when optional field with empty value.
                        var isEmptyValue = !!(yield requiredValidator(value, givenCtx));
                        if (isOptional && isEmptyValue) return resolve();

                        resolve((yield validator(value, givenCtx)));
                    } catch (e) {
                        reject(e);
                    }
                });

                return function (_x11, _x12) {
                    return ref.apply(this, arguments);
                };
            }());
        }));

        var errors = _lodash2.default.filter(results, function (val) {
            return val != null;
        });
        if (errors.length > 0) {
            return errors;
        }

        return;
    });

    return function validateValue(_x8, _x9, _x10) {
        return ref.apply(this, arguments);
    };
}();